import { VNScene } from "./vnscene.ts";
import { Thrall } from "./thralls.ts";

export type Stat = "AGI" | "INT" | "CHA" | "PSI";
export const ALL_STATS: Array<Stat> = ["AGI", "INT", "CHA", "PSI"];

export type Resource = "EXP";
export const ALL_RESOURCES: Array<Resource> = ["EXP"];

export type SkillGoverning = {
  stats: Stat[];
  underTarget: number;
  target: number;
  cost: number;
  note: string;
  scoring: SkillScoring;
  mortalServantValue: number;
  flipped: boolean;
};
export type SkillProfile = {
  name: string;
  description: string;
};

export type SkillData = {
  isDegrading?: boolean;
  governing: SkillGoverning;
  profile: SkillProfile;
  prereqs: Skill[];
};

export type ScoringCategory =
  | "bat"
  | "stealth"
  | "charm"
  | "stare"
  | "party"
  | "lore";
export const SCORING_CATEGORIES: ScoringCategory[] = [
  "bat",
  "stealth",
  "charm",
  "stare",
  "party",
  "lore",
];
export type SkillScoring = { [P in ScoringCategory]?: number };

export type Skill = {
  id: number;
};

export type WishData = {
  profile: {
    name: string;
    note: string;
    domicile: string;
    reignSentence: string;
    failureName: string;
    failureDomicile: string;
    failureReignSentence: string;
    failureSuccessorVerb: string;
  };
  isRandomlyAvailable: boolean;
  isCompulsory: boolean;
  bannedSkills: () => Skill[];
  discouragedSkills: () => Skill[];
  encouragedSkills: () => Skill[];
  requiredSkills: () => Skill[];
  prologue: VNScene;
  onVictory: VNScene;
  onFailure: VNScene;
};
export type Wish = {
  id: number;
};

// endings

export type Ending = {
  scene: VNScene;
  personal: EndingPersonal;
  analytics: EndingAnalytics;
  successorOptions: SuccessorOption[];
  wishOptions: Wish[];

  // forcedSuccessors: number[] | null,
  // forcedWishes: number[] | null
};

export type EndingPersonal = {
  rank: string;
  domicile: string;
  reignSentence: string;
  successorVerb: string;
  progenerateVerb: string;
};

export type EndingAnalytics = {
  itemsPurloined: number;
  vampiricSkills: number;
  mortalServants: number;
};

export type SuccessorOption = {
  name: string;
  title: string;
  template: Thrall;
  note: string | null; // ex "already a vampire"
  stats: Record<Stat, number>;
  talents: Record<Stat, number>;
  skills: Skill[];
  inPenance: boolean;
  isCompulsory: boolean;
};
